@@ -185,8 +185,8 @@ class _TreePenaltyAssigner(pytree_visitor.PyTreeVisitor):
             #   atom tr1() tr2
             # It may be necessary (though undesirable) to split up a previous
             # function call's parentheses to the next line.
-            self._SetUnbreakable(prev_trailer.children[-1])
-          self._SetUnbreakable(cur_trailer.children[0])
+            self._SetStronglyConnected(prev_trailer.children[-1])
+          self._SetStronglyConnected(cur_trailer.children[0])
           prev_trailer_idx = cur_trailer_idx
         else:
           break
@@ -209,7 +209,8 @@ class _TreePenaltyAssigner(pytree_visitor.PyTreeVisitor):
           if last_child_node.value.strip().startswith('#'):
             last_child_node = last_child_node.prev_sibling
           if not style.Get('DEDENT_CLOSING_BRACKETS'):
-            self._SetUnbreakable(last_child_node)
+            if _LastChildNode(last_child_node.prev_sibling).value != ',':
+              self._SetUnbreakable(last_child_node)
 
           if _FirstChildNode(trailer).lineno == last_child_node.lineno:
             # If the trailer was originally on one line, then try to keep it
